import os
from path import Path
from subprocess import check_output

from charmhelpers.core import hookenv
from charmhelpers.core import unitdata
from jujubigdata import utils
from jujubigdata.relations import Spark


# Extended status support
# We call update_blocked_status from the "requires" section of our service
# block, so be sure to return True. Otherwise, we'll block the "requires"
# and never move on to callbacks. The other status update methods are called
# from the "callbacks" section and therefore don't need to return True.
def update_blocked_status():
    if unitdata.kv().get('charm.active', False):
        return True
    spark = Spark()
    if not spark.is_ready():
        hookenv.status_set('waiting', 'Waiting for Spark to become ready')
    return True


def update_working_status():
    if unitdata.kv().get('charm.active', False):
        hookenv.status_set('maintenance', 'Updating configuration')
        return
    hookenv.status_set('maintenance', 'Setting up Spark Notebook')


def update_active_status():
    unitdata.kv().set('charm.active', True)
    hookenv.status_set('active', 'Ready')


# Main Notebook class for callbacks
class Notebook(object):
    def __init__(self, dist_config):
        self.dist_config = dist_config

    def is_installed(self):
        return unitdata.kv().get('notebook.installed')

    def install(self, force=False):
        if not force and self.is_installed():
            return
        self.dist_config.add_dirs()
        self.dist_config.add_packages()

        # Copy our start/stop scripts (preserving attrs) to $HOME
        start_source = 'scripts/start_notebook.sh'
        Path(start_source).chmod(0o755)
        Path(start_source).chown('ubuntu', 'hadoop')

        stop_source = 'scripts/stop_notebook.sh'
        Path(stop_source).chmod(0o755)
        Path(stop_source).chown('ubuntu', 'hadoop')

        target = os.environ.get('HOME', '/home/ubuntu')
        Path(start_source).copy2(target)
        Path(stop_source).copy2(target)

        # Create an IPython profile
        utils.run_as("ubuntu", 'ipython', 'profile', 'create', 'pyspark')

        unitdata.kv().set('notebook.installed', True)

    def configure_notebook(self):
        # profile config created during install
        ipython_profile = "ipython_notebook_config.py"
        # find path to ipython_notebook_config.py
        pPath = "/home/ubuntu/.ipython/profile_pyspark"
        cmd = "find {} -name {}".format(pPath, ipython_profile)
        ipython_profile_path = check_output(cmd.split()).strip()

        # update profile with standard opts and configured port
        utils.re_edit_in_place(ipython_profile_path, {
            r'.*c.NotebookApp.ip *=.*': 'c.NotebookApp.ip = "*"',
            r'.*c.NotebookApp.open_browser *=.*': 'c.NotebookApp.open_browser = False',
            r'.*c.NotebookApp.port *=.*': 'c.NotebookApp.port = {}'.format(self.dist_config.port('notebook')),
            r'.*c.NotebookManager.notebook_dir *=.*': "c.NotebookManager.notebook_dir = u'{}'".format(self.dist_config.path('notebooks')),
        })

        spark_home = os.environ.get("SPARK_HOME", '/usr/lib/spark')
        py4j = "py4j-0.*.zip"
        cmd = "find {} -name {}".format(spark_home, py4j)
        # TODO: handle missing py4j
        py4j_path = check_output(cmd.split()).strip()

        setup_source = 'scripts/00-pyspark-setup.py'
        Path(setup_source).chmod(0o755)
        Path(setup_source).chown('ubuntu', 'hadoop')
        utils.re_edit_in_place(setup_source, {
            r'py4j *=.*': 'py4j="{}"'.format(py4j_path),
        })
        setup_target = '{}/.ipython/profile_pyspark/startup/00-pyspark-setup.py'.format(os.environ.get('HOME', '/home/ubuntu'))
        Path(setup_source).copy2(setup_target)

        # Our spark charm defaults to yarn-client, so that should
        # be a safe default here in case MASTER isn't set. Update the env
        # with our spark mode and py4j location.
        spark_mode = os.environ.get("MASTER", "yarn-client")
        with utils.environment_edit_in_place('/etc/environment') as env:
            env['PYSPARK_DRIVER_PYTHON_OPTS'] = "notebook"
            env['PYSPARK_SUBMIT_ARGS'] = "--master " + spark_mode
            env['PYTHONPATH'] = "{}:{}/python".format(py4j_path, os.environ.get("SPARK_HOME", "/usr/lib/spark"))

    def start(self):
        self.stop()
        script_path = os.environ.get('HOME', '/home/ubuntu')+'/start_notebook.sh'
        # TODO: check for executable script; error without it
        utils.run_as("ubuntu", script_path)

    def stop(self):
        script_path = os.environ.get('HOME', '/home/ubuntu')+'/stop_notebook.sh'
        # TODO: check for executable script; error without it
        utils.run_as("ubuntu", script_path)

    def cleanup(self):
        ipython_profile = Path("/home/ubuntu/.ipython")
        ipython_profile.rmtree()
        with utils.environment_edit_in_place('/etc/environment') as env:
            env['PYSPARK_DRIVER_PYTHON_OPTS'] = ""
